# -*- coding: utf-8 -*-
"""
	ExtendedConfigParser, released under the zlib/libpng license
    Modified version with a SchemaConfigParser and some code cleanup (2008-07-14)

	Copyright (c) 2006-2008 Friedrich Weber

	This software is provided 'as-is', without any express or implied
	warranty. In no event will the authors be held liable for any damages
	arising from the use of this software.
	
	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	    1. The origin of this software must not be misrepresented; you must not
	    claim that you wrote the original software. If you use this software
	    in a product, an acknowledgment in the product documentation would be
	    appreciated but is not required.
	
	    2. Altered source versions must be plainly marked as such, and must not be
	    misrepresented as being the original software.
	
	    3. This notice may not be removed or altered from any source
	    distribution.

"""

import os.path
from ConfigParser import ConfigParser

class ExtendedConfigParser(ConfigParser):
    """ configuration file reader class """
    def __init__(self):
        ConfigParser.__init__(self)
        self.loadedfile = ''

    def read(self, filename):
        """ read one file; be quiet if it does not exist """
        if os.path.exists(filename):
            ConfigParser.read(self, [filename])
        self.loadedfile = filename

    def save(self, filename=None):
        """ save the configuration to `filename` or the file we read in (self.loadedfile) """
        if filename is None:
            filename = self.loadedfile
        f = file(filename, 'w')
        self.write(f)
        f.close

    def set(self, section, option, value):
        """ set a value and create the section if not existing """
        if not self.has_section(section):
           self.add_section(section)
        ConfigParser.set(self, section, option, str(value))
        
    def get(self, section, option, default=None):
        """ return the option `option` in section `section`, if not existing, return `default` """
        return ConfigParser.get(self, section, option) if (self.has_section(section) and self.has_option(section, option)) else default

class SchemaConfigParser(ExtendedConfigParser):
    """ a config parser which converts every value to its type.
        Example:
            parser = SchemaConfigParser({'Section': {'Option1': int} })
            parser.get('Section', 'Option1') -> will return an int
    """
    def __init__(self, schema):
        ExtendedConfigParser.__init__(self)
        self.schema = schema

    def _get_type_of(self, section, option, default=str):
        """ return the type of the option `option` in `section` """
        if section in self.schema:
            if option in self.schema[section]:
                return self.schema[section][option]
        return default

    def get(self, section, option, default=None):
        """ return it in the right type, specified by the scheme """
        ret = ExtendedConfigParser.get(self, section, option, default)
        return self._get_type_of(section, option)(ret)
