# -*- coding: utf-8 -*-
"""
    gestikk, version 0.7

    gestikk provides mouse gestures for you.
    Copyright (C) 2007-2008 Friedrich Weber, Jonas Haag

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

import pygtk
pygtk.require("2.0")
import gtk
import os

from iconhelper import ICON_HELPER

_ = lambda x:x # REALLY TODO

class BaseAction(object):
    """ Prototype for all executable actions. It has
        some placeholders for information and functions.
        Hopefully gestikk will be easily extensible with actions.
    """
    name = ''
    shortname = ''
    def __init__(self, gesture):
        """
            :param gesture: The gestikk.gestures.Gesture object
        """
        self.gesture = gesture
        self.condition = ''

    def execute(self):
        """ execute the action. """
        pass

    def human_readable(self):
        """ return the action as human-readable string. """
        return ''

    def __repr__(self):
        return '<Action: %s>' % self.human_readable()

    def write_to_node(self, node):
        """ write all information (except the gesture) to a xml.etree.ElementTree.SubElement node. """
        node.set('condition', self.condition)

    def get_from_node(self, node):
        """ get all information (except the gesture) from the node """
        self.condition = node.get('condition', '')

    def get_option_widget(self):
        """ return a widget which contains other widgets which let the user set the action options.
            Fill in all options.
            """
        pass

    def handle_option_widget(self, widget):
        """ Fill in the options from the widget. It is the same that you returned in `get_option_widget`. """
        pass

class CommandAction(BaseAction):
    """ This action executes a specific command. """
    name = 'Command action'
    shortname = 'command'

    def __init__(self, gesture):
        BaseAction.__init__(self, gesture)
        self.command = ''

    def get_icon(self):
        return ICON_HELPER.lookup_icon(self.command, 48)

    def execute(self):
        """ execute the action. """
        pass

    def human_readable(self):
        """ return the action as human-readable string. """
        return _('Start application "%s"') % self.command

    def write_to_node(self, node):
        """ write the command to node """
        BaseAction.write_to_node(self, node)
        node.set('command', self.command)

    def get_from_node(self, node):
        """ get the command from node """
        BaseAction.get_from_node(self, node)
        self.command = node.get('command', '')

    def get_option_widget(self):
        """ return the command entry """
        entry = gtk.Entry()
        entry.set_text(self.command)
        return entry

    def handle_option_widget(self, widget):
        """ handle it. """
        self.command = widget.get_text()
        print self
        
# some magic code to get all action subclasses in a dict {Class.shortname:Class}
ACTIONS = dict([(action.shortname, action) for action in locals().values() if hasattr(action, 'shortname') and issubclass(action, BaseAction)])

CommandAction((-1, 0))

output = False #TEMP
if __name__ == "__main__" and output:

    print "This code doesn't do anything when you run it that way, you idiot!"
    print "Anyways, I'll tell you which action classes I do contain:"
    print

    print "| %-15s | %-40s|" % ("Shortname", "Action class")
    print "%s%s%s%s%s" %("|", "="*17, "|", "="*41, "|")
    for shortname in ACTIONS:
        print "| %-15s | %-40s|" % (shortname, ACTIONS[shortname])
