# -*- coding: utf-8 -*-
"""
    gestikk, version 0.7

    gestikk provides mouse gestures for you.
    Copyright (C) 2007-2008 Friedrich Weber, Jonas Haag

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

import gtk

from conditions import evaluate_condition
from actionmaster import get_all_actions
from gtkwizard import WizardPage, WizardDialog
from gtkwidgets import EasyComboBox

class ActionOptionPage(WizardPage):
    def __init__(self, wizard, action):
        self.action = action
        WizardPage.__init__(self, wizard, action.get_option_widget())

    def page_done(self):
        self.action.handle_option_widget(self.widget)
        self.wizard.add_page(ActionConditionPage(self.wizard, self.action))

class ActionChooserPage(WizardPage):
    """ an page which instantiates an action - self.action """
    def __init__(self, wizard):
        self.action = None
        
        vbox = gtk.VBox()
        combo = EasyComboBox()
        vbox.pack_start(combo, True, True)
        for action in get_all_actions().itervalues():
            combo.append(action.name, action)
        vbox.combo = combo
        vbox.show_all()

        WizardPage.__init__(self, wizard, vbox)

    def page_done(self):
        """ The page is done. set wizards action to an instance of the chosen action """
        Klass = self.widget.combo.get_selected_user_data()
        action = Klass(self.wizard.gesture)
        self.wizard.action = action
        # add the right action option page to the wizard
        self.wizard.add_page(ActionOptionPage(self.wizard, action))

class ActionConditionPage(WizardPage):
    """ a page for the conditions. """
    def __init__(self, wizard, action):
        self.action = action
        # create the widget...
        vbox = gtk.VBox()
        # condition entry
        vbox.cond_entry = gtk.Entry()
        vbox.pack_start(vbox.cond_entry, True, True)
        # 'test it'
        vbox.test_it = gtk.Button('Test it') # TODO: gettext
        vbox.pack_start(vbox.test_it, True, True)
        vbox.test_it.connect('clicked', self.sig_test)
        WizardPage.__init__(self, wizard, vbox)

    def dry_test(self):
        """ do a little dry test.
            :returns: True if syntax is correct, False if it isn't
        """
        evaluate_condition(self.widget.cond_entry.get_text())
        return True

    def sig_test(self, widget):
        if self.dry_test():
            print evaluate_condition(self.widget.cond_entry.get_text())

    def page_done(self):
        self.action.condition = self.widget.cond_entry.get_text()

class ActionWizard(WizardDialog):
    def __init__(self, gesture, action=None):
        """ 
            :param gesture: gesture as string
            :param action: The action OR None -> display action kind chooser
        """
        self.gesture = gesture

        total_pages = 2 # Options - Condition
        if action is None:
            total_pages += 1 # plus Action Chooser
        WizardDialog.__init__(self, None, 'New action', total_pages) # TODO: gettext
        if action is None:
            self.add_page(ActionChooserPage(self))
        else:
            self.action = action

if __name__ == '__main__':
    d = ActionWizard('(1,0)')
    d.run()
