# -*- coding: utf-8 -*-
"""
    gestikk, version 0.7

    gestikk provides mouse gestures for you.
    Copyright (C) 2007-2008 Friedrich Weber, Jonas Haag

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

import sys
import os.path

def get_basedir():
    """ func to get the basedir, from exaile """
    basedir = os.path.dirname(os.path.realpath(__file__))
    if not os.path.exists(os.path.join(basedir, "gestikk.py")):
        if os.path.exists(os.path.join(os.getcwd(), "gestikk.py")):
            basedir = os.getcwd()
        elif os.path.exists(os.path.join(os.path.normpath(os.path.join(basedir, '..')), 'gestikk.py')):
            basedir = os.path.normpath(os.path.join(basedir, '..'))
        else:
            basedir = '/usr/share/gestikk'
    return basedir

BASEDIR = get_basedir()
sys.path.insert(0, BASEDIR)

def get_path_for(kind):
    """ return path for `kind`; linux-only at the moment
        :param kind: one of 'pixmap', 'icon'
    """
    return {'pixmap':'/usr/share/pixmaps',
            'icon':'/usr/share/icons',
            'gladefile':'/usr/share/gestikk',
            'executable':'/usr/bin',
            'configuration':os.path.expanduser('~/.gestikk')}.get(kind, '')

def get_file(name, kind):
    """ get a file. I'll try to search it in-place, then system-wide; linux-only at the moment
        :param kind: see get_path
    """
    if os.path.exists(os.path.join(BASEDIR, name)): # in-place
        return os.path.join(BASEDIR, name)
    elif os.path.exists(os.path.join(BASEDIR, 'media', name)):
        return os.path.join(BASEDIR, 'media', name)
    elif os.path.exists(os.path.join(os.path.dirname(__file__), name)):
        return os.path.join(os.path.dirname(__file__), name)
    else: # system-wide
        path = get_path_for(kind)
        if not path or not os.path.exists(os.path.join(path, name)):
            raise Exception("Could not find file '%s' which should be a %s. Please report a bug." % (name, kind))
            return ''
        else:
            return os.path.join(path, name)
