# -*- coding: utf-8 -*-
"""
    gestikk, version 0.7

    gestikk provides mouse gestures for you.
    Copyright (C) 2007-2008 Friedrich Weber, Jonas Haag

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

import pygtk
pygtk.require("2.0")
import gtk

def gesture2pixbuf(pixmap):
    return pixmap2pixbuf(pixmap, (255, 255, 255))

def gesture2pixmap(gesture, size = 20, color= '#0000FF'):
    return gesturetuple2pixmap(strtuple2pytuple(gesture), size, color)

def _split_gesture(s):
    """ split gesture by ,; return list of string-tuples """
    l = []
    old = ''
    in_tuple = False
    for char in s:
        if char == '(':
            old += char
            in_tuple = True
        elif char == ')':
            old += char
            in_tuple = False
        elif char == ',' and not in_tuple:
            l.append(old)
            old = ''
        else:
            old += char
    if old:
        l.append(old)
    return l

def _str2tuple(s):
    """Convert tuple-like strings to real tuples.
        e.g. '(1,2,3,4)' -> (1, 2, 3, 4)

        from http://www.dbforums.com/showpost.php?&p=5469452&postcount=15 - thanks :-)
    """
    if s[0] + s[-1] != "()":
        raise ValueError("Badly formatted string (missing brackets).")
    items = s[1:-1] # removes the leading and trailing brackets
    items = items.split(',')
    L = [int(x.strip()) for x in items] # clean up spaces, convert to ints
    return tuple(L)

def pytuple2strtuple(pyt):
    """ make [(1,0), (0,1)] -> "(1,0),(0,1)" """
    return str(pyt)[1:-1]

def strtuple2pytuple(strt):
    """ make "(1,0),(0,1)" -> [(1,0), (0, 1)] """
    return [_str2tuple(st) for st in _split_gesture(strt)]

def gesturetuple2pixmap(gesture, size, color): # gestikk 0.6
    """ create nice pixmap out of gesture tuple """
    point0, point9 = _get_09(gesture, size)
    point = (-point0[0]+size//3, -point0[1]+size//3)
    img_size = (int(point9[0]+(size*(4/5))), int(point9[1]+(size*(4/5))))
    color = gtk.gdk.color_parse(color)
    bg = gtk.gdk.color_parse('#FFFFFF')
    pixmap = gtk.gdk.Pixmap(None, img_size[0], img_size[1], 24)
    gc = gtk.gdk.GC(pixmap)
    gc.set_rgb_fg_color(color)
    gc.set_rgb_bg_color(bg)
    gc.set_line_attributes(2, gtk.gdk.LINE_SOLID, gtk.gdk.CAP_NOT_LAST, gtk.gdk.JOIN_MITER)

    # clear
    gc.set_rgb_fg_color(bg)
    pixmap.draw_rectangle(gc, True, 0, 0, img_size[0], img_size[1])
    gc.set_rgb_fg_color(color)

    # draw the start point
    ps = max(size // 35, 1)
    pixmap.draw_rectangle(gc, True, point[0]-ps, point[1]-ps, 2*ps, 2*ps)

    first = True
    for g in gesture:
        if first:
            new_point = (point[0]+g[0]*size-10, point[1]+g[1]*size)
        else:
            new_point = (point[0]+g[0]*size, point[1]+g[1]*size)
        pixmap.draw_lines(gc, [point, new_point])
        point = new_point
        first = False
        # set the color
    return pixmap

def pixmap2pixbuf(pm, transparent_color=None): # gestikk 0.6
    """ convert a pixmap to pixbuf
        :param transparent_color: tripel of rgb or None
    """
    pw, ph = pm.get_size()
    pb = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, False, 8, pw, ph)
    pb.get_from_drawable(pm, pm.get_colormap(), 0, 0, 0, 0, pw, ph)
    if transparent_color:
        transparent_color = [chr(t) for t in transparent_color]
        pb = pb.add_alpha(True, transparent_color[0], transparent_color[1], transparent_color[2])
    return pb

def _get_09(gesture, size=150): # gestikk 0.6
    """ calculate the 0,0 point """
    point0 = [0, 0] # the zero point
    point9 = [0, 0] # the last point
    # imagine a rectangle:
    # P0 ------------------------
    # |                          |
    # |                          |
    # --------------------------P9
    # P0: point0
    # P9: point9
    # ... easy, isn't it?
    point = point0[:]
    first = True
    for g in gesture:
        if first:
            new_point = (point[0]+g[0]*size-10, point[1]+g[1]*size)
        else:
            new_point = (point[0]+g[0]*size, point[1]+g[1]*size)
        # Set Point0
        if new_point[0] < point0[0]:
            point0[0] = new_point[0]
        if new_point[1] < point0[1]:
            point0[1] = new_point[1]
        # Set Point9
        if abs(new_point[0]) > point9[0]:
            point9[0] = abs(new_point[0])
        if abs(new_point[1]) > point9[1]:
            point9[1] = abs(new_point[1])
        point = new_point
        first = False
    return point0, point9
