# -*- coding: utf-8 -*-
"""
    gestikk, version 0.7

    gestikk provides mouse gestures for you.
    Copyright (C) 2007-2008 Friedrich Weber, Jonas Haag

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

import gtk
import gobject
import gesture

class EasyListView(gtk.TreeView):
    """ Some easy ListView (or whatever) Widget """

    def __init__(self, columns_labels = {0 : ["Label", "STRING"]}):
        """
            :param: columns_labels is a dict like this: {0 : ["Label", "STRING"]}
        """
        self.columns = {}

        self.treestore = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
        gtk.TreeView.__init__(self, self.treestore)

        for i in range(len(columns_labels)):
            self.columns[i] = [gtk.TreeViewColumn(columns_labels[i][0]), gtk.CellRendererText()]
            column = self.columns[i]
            column[0].pack_start(column[1], True)
            gtk.TreeView.append_column(self, column[0])
            column[0].add_attribute(column[1], 'text', i)

        gtk.TreeView.show(self)

    def add_row(self, data, parent = None):
        return self.treestore.append(parent, data)


class GestureTreeView(gtk.TreeView):
    def __init__(self):
        self.treestore = gtk.TreeStore( gtk.gdk.Pixbuf, gobject.TYPE_STRING,    gobject.TYPE_STRING)
                                        # gesture icon  # gesture action        # gesture condition
        # create TreeStore widget to store data in it

        # treeview widget to display data:
        gtk.TreeView.__init__(self, self.treestore)

        # add columns:
        self.pixbufcolumn = gtk.TreeViewColumn()
        self.actioncolumn = gtk.TreeViewColumn('Action')
        self.conditioncolumn = gtk.TreeViewColumn('Condition')

        # create renderer widgets to render data in columns:
        self.pixbufrenderer = gtk.CellRendererPixbuf()
        self.actiontextrenderer = gtk.CellRendererText()
        self.conditiontextrenderer = gtk.CellRendererText()

        # add renderer to columns:
        self.pixbufcolumn.pack_start(self.pixbufrenderer, True)
        self.actioncolumn.pack_start(self.actiontextrenderer, True)
        self.conditioncolumn.pack_start(self.actiontextrenderer, True)

        # add columns to TreeView:
        gtk.TreeView.append_column(self, self.pixbufcolumn)
        gtk.TreeView.append_column(self, self.actioncolumn)
        gtk.TreeView.append_column(self, self.conditioncolumn)

        # set data columns:
        self.pixbufcolumn.add_attribute(self.pixbufrenderer, 'pixbuf', 0)
        self.actioncolumn.add_attribute(self.actiontextrenderer, 'text', 1)
        self.conditioncolumn.add_attribute(self.actiontextrenderer, 'text', 2)

        # add treeview
        gtk.TreeView.show(self)

    def pixbuf_(self, target):
        try:
            return gtk.gdk.pixbuf_new_from_file(target)
        except:
            return False

    def add_row(self, data, parent = None):
        return self.treestore.append(parent, data)


class EasyComboBox(gtk.ComboBox):
    """ an easy combobox which stores user data for every string item """
    def __init__(self):
        self.store = gtk.ListStore(str)
        gtk.ComboBox.__init__(self, self.store)
        cell = gtk.CellRendererText()
        self.pack_start(cell, True)
        self.add_attribute(cell, 'text', 0)
        # { Item index: User Data }
        self.data = []
        # [Item, Item, ...]
        self.items = []

    def append(self, text, data=None):
        """ append an item. `text` is the text, `data` the data. """
        self.store.append([text])
        self.items.append(text)
        self.data.append(data)

    def get_selected_user_data(self):
        """ return the user data of the selected item """
        active = self.get_active()
        if active < 0:
            return None
        return self.data[active]

if __name__ == '__main__' and 10 == 5:
    class Fi(object):
        def __init__(self, a):
            self.a = a

        def __repr__(self):
            return '<fi %s>' % self.a
    import sys
    x = gtk.Window()
    e = EasyComboBox()
    x.add(e)
    e.show()
    e.append('Blubb', Fi('Blubb'))
    e.append('Bla', Fi('Bla'))
    e.connect('changed', lambda w: sys.stdout.write(str(w.get_selected_user_data())+"\n") and sys.stdout.flush())
    x.show()
    gtk.main()

elif __name__ == "__main__":
    # haha!1
    hannes = EasyListView({0: ["Hans", "STRING"], 1 : ["Peter", "STRING"]})
    for i in range(3):
        pa = hannes.add_row(["Deine Mama", "ist ein Kuchen"])
        for j in range(2):
            hannes.add_row(["und auch", "ein Kartoffelsalat"], pa)
    win = gtk.Window()
    win.add(hannes)
    win.show()
    gtk.main()