# -*- coding: utf-8 -*-
"""
    gestikk, version 0.7

    gestikk provides mouse gestures for you.
    Copyright (C) 2007-2008 Friedrich Weber, Jonas Haag

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

import gtk

class WizardPage(object):
    def __init__(self, wizard, widget):
        self.widget = widget
        self.wizard = wizard

    def get(self):
        return self.widget

    def page_done(self):
        pass

class WizardDialog(gtk.Dialog):
    def __init__(self, parent, title, total_pages):
        gtk.Dialog.__init__(self, title, parent)

        self.current_page_idx = 0
        self.current_page = None
        self._current_widget = None
        self.pages = []
        self.total_pages = total_pages

        self.btn_previous = gtk.Button(None, gtk.STOCK_MEDIA_PREVIOUS)
        self.action_area.pack_start(self.btn_previous)
        self.btn_previous.connect('clicked', self.signal_previous)
        self.btn_previous.show()
        self.btn_previous.set_use_stock(True)

        self.btn_next = gtk.Button(None, gtk.STOCK_MEDIA_NEXT)
        self.action_area.pack_start(self.btn_next)
        self.btn_next.connect('clicked', self.signal_next)
        self.btn_next.show()
        self.btn_next.set_use_stock(True)

    def add_page(self, page):
        """ add a page. `page` should be an instance of WizardPage. """
        self.pages.append(page)

    def show_page(self):
        """ show the current page """
        if not self._current_widget is None:
            self.vbox.remove(self._current_widget)
        self.current_page = self.pages[self.current_page_idx]
        self._current_widget = self.current_page.get()
        self._current_widget.show_all()
        self.vbox.pack_start(self._current_widget, True, True)

        # do buttons
        if self.current_page_idx == self.total_pages - 1:
            self.btn_next.set_label(gtk.STOCK_OK)
        else:
            self.btn_next.set_label(gtk.STOCK_MEDIA_NEXT)
        if self.current_page_idx == 0:
            self.btn_previous.set_sensitive(False)
        else:
            self.btn_previous.set_sensitive(True)


    def signal_next(self, widget):
        self.current_page.page_done()
        self.current_page_idx += 1
        if self.current_page_idx >= self.total_pages:
            self.finished()
        else:
            self.show_page()

    def finished(self):
        self.response(gtk.RESPONSE_OK)

    def signal_previous(self, widget):
        self.current_page_idx -= 1
        self.show_page()

    def run(self):
        self.show_page()
        gtk.Dialog.run(self)

if __name__ == '__main__':
    wd = WizardDialog(None, 'Blubb', 2)

    b = gtk.Button('Hallo')
    wd.add_page(WizardPage(wd, b))

    b = gtk.Button('Tschö')
    wd.add_page(WizardPage(wd, b))

    wd.run()
    gtk.main()

