# -*- coding: utf-8 -*-
"""
    gestikk, version 0.7

    gestikk provides mouse gestures for you.
    Copyright (C) 2007-2008 Friedrich Weber, Jonas Haag

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
"""

try:
    import xml.etree.cElementTree as ETree
except ImportError:
    import xml.etree.ElementTree as ETree

import actions
import gesture

def indent_tree(elem, level=0):
    """ indent a tree. taken from http://effbot.org/zone/element-lib.htm#prettyprint """
    i = "\n" + level*"  "
    if len(elem):
        if not elem.text or not elem.text.strip():
            elem.text = i + "  "
        for elem in elem:
            indent_tree(elem, level+1)
        if not elem.tail or not elem.tail.strip():
            elem.tail = i
    else:
        if level and (not elem.tail or not elem.tail.strip()):
            elem.tail = i

def store_gestures(gestures):
    """ return the gestures as indented xml
        :param gestures: A gestures dictionary {gesture as string: [Action, Action]}
    """
    root = ETree.Element('gestikk')
    for gesture in gestures.iterkeys():
        gesture_node = ETree.SubElement(root, 'gesture')
        gesture_node.set('gesture', gesture)
        for action in gestures[gesture]:
            action_node = ETree.SubElement(gesture_node, 'action')
            action_node.set('type', action.__class__.shortname)
            action.write_to_node(action_node)
    # make it pretty
    indent_tree(root)
    return ETree.tostring(root)

def read_gestures(string):
    """ get the XML from string `string` and return a complete dictionary """
    mygestures = {}
    root = ETree.fromstring(string)
    for gesture_node in root.getchildren():
        mygesture = gesture_node.get('gesture')
        assert mygesture
        mygestures[mygesture] = []
        for action_node in gesture_node.getchildren():
            action_class = actions.ACTIONS[action_node.get('type')] # get the action class
            action = action_class(gesture) # and instantiate
            action.get_from_node(action_node)
            mygestures[mygesture].append(action)
    return mygestures

if __name__ == '__main__':
    print read_gestures("""
    <gestikk>
        <gesture gesture="(1,0),(0,1)">
            <action type="command" command="Blubb" />
        </gesture>
    </gestikk>
    """)
